﻿namespace TabsStudioSync
{
    class SolutionExplorer
    {
        public SolutionExplorer(EnvDTE80.DTE2 dte_)
        {
            dte = dte_;
        }

        public void SyncDocument(EnvDTE.Document doc)
        {
            ActivateSolutionExplorer();
            
            EnvDTE.ProjectItem projectItem = doc.ProjectItem;
            if (projectItem == null)
                return;

            EnvDTE.UIHierarchyItem target = Expand(projectItem);
            target.Select(EnvDTE.vsUISelectionType.vsUISelectionTypeSelect);
        }

        public void SyncDocuments(System.Collections.Generic.List<EnvDTE.Document> docs)
        {
            ActivateSolutionExplorer();

            bool firstItem = true;
            foreach (EnvDTE.Document doc in docs)
            {
                EnvDTE.ProjectItem projectItem = doc.ProjectItem;
                if (projectItem == null)
                    continue;

                EnvDTE.UIHierarchyItem target = Expand(projectItem);
                if (firstItem)
                {
                    target.Select(EnvDTE.vsUISelectionType.vsUISelectionTypeSelect);
                    firstItem = false;
                }
                else
                {
                    if (!target.IsSelected)
                        target.Select(EnvDTE.vsUISelectionType.vsUISelectionTypeToggle);
                }
            }
        }

        private EnvDTE.UIHierarchyItem Expand(EnvDTE.ProjectItem projectItem)
        {
            return ExpandPath(dte.ToolWindows.SolutionExplorer.UIHierarchyItems.Item(1).UIHierarchyItems, GetPath(projectItem));
        }

        private void ActivateSolutionExplorer()
        {
            dte.ToolWindows.SolutionExplorer.Parent.Activate();
        }

        private System.Collections.Generic.List<object> GetPath(object p)
        {
            if (p == null)
                return null;

            System.Collections.Generic.List<object> result = new System.Collections.Generic.List<object>();
            if (p is EnvDTE.ProjectItem)
            {
                EnvDTE.ProjectItem projectItem = p as EnvDTE.ProjectItem;
                if (projectItem.Collection != null)
                {
                    System.Collections.Generic.List<object> parents = GetPath(projectItem.Collection.Parent);
                    if (parents != null)
                        result = parents;
                }
            }
            if (p is EnvDTE.Project)
            {
                EnvDTE.Project project = p as EnvDTE.Project;
                System.Collections.Generic.List<object> parents = GetPath(project.ParentProjectItem);
                if (parents != null)
                    result = parents;
            }
            result.Add(p);
            return result;
        }

        private EnvDTE.UIHierarchyItem ExpandPath(EnvDTE.UIHierarchyItems root, System.Collections.Generic.List<object> path)
        {
            EnvDTE.UIHierarchyItem result = null;
            EnvDTE.UIHierarchyItems currentRoot = root;
            for (int i = 0; i < path.Count; ++i)
            {
                foreach (EnvDTE.UIHierarchyItem h in currentRoot)
                {
                    if (h.Object == path[i])
                    {
                        if (i == path.Count - 1)
                        {
                            result = h;
                        }
                        else
                        {
                            h.UIHierarchyItems.Expanded = true;
                            currentRoot = h.UIHierarchyItems;
                        }
                        break;
                    }
                }
            }
            return result;
        }

        private EnvDTE80.DTE2 dte;
    }
}
