﻿namespace TabsStudioSync
{
    public class Sync : TabsStudioExt.ITabsStudioAddin, EnvDTE.IDTCommandTarget
    {
        public Sync()
        {
            System.Windows.Controls.Image tabImage = new System.Windows.Controls.Image();
            tabImage.Source = new System.Windows.Media.Imaging.BitmapImage(
                new System.Uri("pack://application:,,,/Sync;component/sychronize.png"));

            System.Windows.Controls.Image tabExtensionImage = new System.Windows.Controls.Image();
            tabExtensionImage.Source = new System.Windows.Media.Imaging.BitmapImage(
                new System.Uri("pack://application:,,,/Sync;component/sychronize.png"));

            syncTabExtensionMenuItem = new System.Windows.Controls.MenuItem();
            syncTabExtensionMenuItem.Icon = tabExtensionImage;
            syncTabExtensionMenuItem.Header = "Sync document with Solution Explorer";
            syncTabExtensionMenuItem.Click += SyncTabExtension;

            syncTabMenuItem = new System.Windows.Controls.MenuItem();
            syncTabMenuItem.Icon = tabImage;
            syncTabMenuItem.Header = "Sync tab with Solution Explorer";
            syncTabMenuItem.Click += SyncTab;
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;
            engine.OpeningTabExtensionContextMenu += OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu += OpeningTabContextMenu;

            commandsImpl = new CommandsImpl(engine, dte);
            commands = new Commands(engine, dte, commandsImpl);
        }

        public void OnDisconnection()
        {
            engine.OpeningTabExtensionContextMenu -= OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu -= OpeningTabContextMenu;
        }

        public void QueryStatus(string CmdName, EnvDTE.vsCommandStatusTextWanted NeededText,
                ref EnvDTE.vsCommandStatus StatusOption, ref System.Object CommandText)
        {
            commands.QueryStatus(CmdName, NeededText, ref StatusOption, ref CommandText);
        }

        public void Exec(string CmdName, EnvDTE.vsCommandExecOption ExecuteOption, ref System.Object VariantIn,
            ref System.Object VariantOut, ref bool Handled)
        {
            commands.Exec(CmdName, ExecuteOption, ref VariantIn, ref VariantOut, ref Handled);
        }

        private void OpeningTabExtensionContextMenu(object sender, 
            TabsStudioExt.OpeningTabExtensionContextMenuEventArgs e)
        {
            try
            {
                if (e.TabExtension.Window.Document == null || e.TabExtension.Window.Document.ProjectItem == null)
                    return;
            }
            catch (System.Exception)
            {
                return;
            }
            
            storedTabExtension = e.TabExtension;
            PlaceMenuItem(syncTabExtensionMenuItem, e.ContextMenu);
        }

        private void OpeningTabContextMenu(object sender, TabsStudioExt.OpeningTabContextMenuEventArgs e)
        {
            storedTab = e.Tab;
            PlaceMenuItem(syncTabMenuItem, e.ContextMenu);
        }

        private void SyncTabExtension(object sender, System.Windows.RoutedEventArgs e)
        {
            commandsImpl.SyncExt(storedTabExtension);
        }

        private void SyncTab(object sender, System.Windows.RoutedEventArgs e)
        {
            commandsImpl.SyncTab(storedTab);
        }

        private void PlaceMenuItem(System.Windows.Controls.MenuItem menuItem,
            System.Windows.Controls.ContextMenu menu)
        {
            foreach (object item in menu.Items)
            {
                if (item is System.Windows.Controls.Control)
                {
                    if ((item as System.Windows.Controls.Control).Name.Equals("OpenContainingFolder"))
                    {
                        menu.Items.Insert(menu.Items.IndexOf(item) + 1, menuItem);
                        break;
                    }
                }
            }
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private TabsStudioExt.TabExtension storedTabExtension;
        private System.Windows.Controls.MenuItem syncTabExtensionMenuItem;
        private TabsStudioExt.Tab storedTab;
        private System.Windows.Controls.MenuItem syncTabMenuItem;

        private CommandsImpl commandsImpl;
        private Commands commands;
     }
}